MODULE cpl_rnf
   !!======================================================================
   !!                       ***  MODULE  sbcrnf  ***
   !! Ocean forcing:  river runoff
   !!=====================================================================
   !! History : ?.?  ! 2018-01 (D. Copsey) Initial setup
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   cpl_rnf_init : runoffs initialisation
   !!---------------------------------------------------------------------- 

   USE timing          ! Timing
   USE in_out_manager  ! I/O units
   USE iom
   USE wrk_nemo        ! Memory allocation
   USE dom_oce         ! Domain sizes (for grid box area e1e2t)
   USE cpl_oasis3      ! Coupling information (for n_rivers)
   
   IMPLICIT NONE
   PRIVATE
   
   PUBLIC   cpl_rnf_init     ! routine called in nemo_init
   
   TYPE, PUBLIC ::   RIVERS_DATA     !: Storage for river outflow data
      INTEGER, POINTER, DIMENSION(:,:)    ::   river_number       !: River outflow number
      REAL, POINTER, DIMENSION(:)         ::   river_area         ! 1D array listing areas of each river outflow (m2)
   END TYPE RIVERS_DATA
   
   TYPE(RIVERS_DATA), PUBLIC, TARGET :: rivers  !: River data
   
CONTAINS

   SUBROUTINE cpl_rnf_init
      !!----------------------------------------------------------------------
      !!                    ***  SUBROUTINE cpl_rnf_init  ***
      !!                     
      !! ** Purpose : - Read in file for river outflow numbers
      !!                
      !!
      !!----------------------------------------------------------------------
      !! namelist variables
      !!-------------------
      CHARACTER(len=80)                         ::   file_riv_number             !: Filename for river numbers
      INTEGER                                   ::   ios                 ! Local integer output status for namelist read
      INTEGER                                   ::   inum
      INTEGER                                   ::   ii, jj              !: Loop indices
      INTEGER                                   ::   max_river
      REAL(wp), POINTER, DIMENSION(:,:)         ::   river_number        ! 2D array containing the river outflow numbers
      
      NAMELIST/nam_cpl_rnf/file_riv_number
      !!----------------------------------------------------------------------

      IF( nn_timing == 1 ) CALL timing_start('cpl_rnf_init')
      
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'cpl_rnf_init : initialization of river runoff coupling'
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~'
      
      REWIND(numnam_cfg)
      
      ! Read the namelist
      READ  ( numnam_ref, nam_cpl_rnf, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nam_cpl_rnf in reference namelist', lwp )
      READ  ( numnam_cfg, nam_cpl_rnf, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'nam_cpl_rnf in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, nam_cpl_rnf )

      !                                               ! Parameter control and print
      IF(lwp) WRITE(numout,*) '  '
      IF(lwp) WRITE(numout,*) '          Namelist nam_cpl_rnf : Coupled runoff'
      IF(lwp) WRITE(numout,*) '             Input file that contains river numbers = ',file_riv_number
      IF(lwp) WRITE(numout,*) ' '
      
      ! Assign space for river numbers
      ALLOCATE( rivers%river_number( jpi, jpj ) )
      CALL wrk_alloc( jpi, jpj, river_number )
      
      ! Read the river numbers from netcdf file
      CALL iom_open (file_riv_number , inum )
      CALL iom_get  ( inum, jpdom_data, 'river_number', river_number )
      CALL iom_close( inum )
      
      ! Convert from a real array to an integer array
      max_river=0
      DO ii = 1, jpi
        DO jj = 1, jpj
          rivers%river_number(ii,jj) = INT(river_number(ii,jj))
          
          IF ( rivers%river_number(ii,jj) > max_river ) THEN
            max_river = rivers%river_number(ii,jj)
          END IF
        END DO
      END DO
      
      ! Print out the largest river number
      WRITE(numout,*) 'Maximum river number = ',max_river
      
      ! Get the area of each river outflow
      ALLOCATE( rivers%river_area( n_rivers ) )
      rivers%river_area(:) = 0.0
      DO ii = 1, jpi
        DO jj = 1, jpj
          IF ( rivers%river_number(ii,jj) > 0 .AND. rivers%river_number(ii,jj) <= n_rivers ) THEN
            rivers%river_area(rivers%river_number(ii,jj)) = rivers%river_area(rivers%river_number(ii,jj)) + e1e2t(ii,jj)
          END IF
        END IF
      END IF
      
      ! Use mpp_sum to add together river areas on other processors
      CALL mpp_sum( rivers%river_area )
      WRITE(numout,*) 'Area of river number 1 is ',rivers%river_area(1)
      WRITE(numout,*) 'Area of river number 10 is ',rivers%river_area(10)
      
      
   END SUBROUTINE cpl_rnf_init

END MODULE cpl_rnf
